@@ -2,6 +2,8 @@
 """
 
 import argparse
+import codecs
+import locale
 import logging
 import logging.handlers
 import os
@@ -59,6 +61,7 @@ class App(object):
     LOG_FILE_MESSAGE_FORMAT = \
         '[%(asctime)s] %(levelname)-8s %(name)s %(message)s'
     DEFAULT_VERBOSE_LEVEL = 1
+    DEFAULT_OUTPUT_ENCODING = 'utf-8'
 
     def __init__(self, description, version, command_manager,
                  stdin=None, stdout=None, stderr=None,
@@ -67,12 +70,33 @@ class App(object):
         """
         self.command_manager = command_manager
         self.command_manager.add_command('help', HelpCommand)
-        self.stdin = stdin or sys.stdin
-        self.stdout = stdout or sys.stdout
-        self.stderr = stderr or sys.stderr
+        self._set_streams(stdin, stdout, stderr)
         self.interactive_app_factory = interactive_app_factory
         self.parser = self.build_option_parser(description, version)
         self.interactive_mode = False
+        self.interpreter = None
+
+    def _set_streams(self, stdin, stdout, stderr):
+        locale.setlocale(locale.LC_ALL, '')
+        if sys.version_info[:2] == (2, 6):
+            # Configure the input and output streams. If a stream is
+            # provided, it must be configured correctly by the
+            # caller. If not, make sure the versions of the standard
+            # streams used by default are wrapped with encodings. This
+            # works around a problem with Python 2.6 fixed in 2.7 and
+            # later (http://hg.python.org/cpython/rev/e60ef17561dc/).
+            lang, encoding = locale.getdefaultlocale()
+            encoding = (getattr(sys.stdout, 'encoding', None)
+                        or encoding
+                        or self.DEFAULT_OUTPUT_ENCODING
+                        )
+            self.stdin = stdin or codecs.getreader(encoding)(sys.stdin)
+            self.stdout = stdout or codecs.getwriter(encoding)(sys.stdout)
+            self.stderr = stderr or codecs.getwriter(encoding)(sys.stderr)
+        else:
+            self.stdin = stdin or sys.stdin
+            self.stdout = stdout or sys.stdout
+            self.stderr = stderr or sys.stderr
 
     def build_option_parser(self, description, version,
                             argparse_kwargs=None):
@@ -94,45 +118,45 @@ class App(object):
             description=description,
             add_help=False,
             **argparse_kwargs
-            )
+        )
         parser.add_argument(
             '--version',
             action='version',
             version='%(prog)s {0}'.format(version),
-            )
+        )
         parser.add_argument(
             '-v', '--verbose',
             action='count',
             dest='verbose_level',
             default=self.DEFAULT_VERBOSE_LEVEL,
             help='Increase verbosity of output. Can be repeated.',
-            )
+        )
         parser.add_argument(
             '--log-file',
             action='store',
             default=None,
             help='Specify a file to log output. Disabled by default.',
-            )
+        )
         parser.add_argument(
             '-q', '--quiet',
             action='store_const',
             dest='verbose_level',
             const=0,
             help='suppress output except warnings and errors',
-            )
+        )
         parser.add_argument(
             '-h', '--help',
             action=HelpAction,
             nargs=0,
             default=self,  # tricky
             help="show this help message and exit",
-            )
+        )
         parser.add_argument(
             '--debug',
             default=False,
             action='store_true',
             help='show tracebacks on errors',
-            )
+        )
         return parser
 
     def configure_logging(self):
@@ -145,7 +169,7 @@ class App(object):
         if self.options.log_file:
             file_handler = logging.FileHandler(
                 filename=self.options.log_file,
-                )
+            )
             formatter = logging.Formatter(self.LOG_FILE_MESSAGE_FORMAT)
             file_handler.setFormatter(formatter)
             root_logger.addHandler(file_handler)
@@ -224,16 +248,23 @@ class App(object):
         return
 
     def interact(self):
-        interpreter = self.interactive_app_factory(self,
-                                                   self.command_manager,
-                                                   self.stdin,
-                                                   self.stdout,
-                                                   )
-        interpreter.cmdloop()
+        self.interpreter = self.interactive_app_factory(self,
+                                                        self.command_manager,
+                                                        self.stdin,
+                                                        self.stdout,
+                                                        )
+        self.interpreter.cmdloop()
         return 0
 
     def run_subcommand(self, argv):
-        subcommand = self.command_manager.find_command(argv)
+        try:
+            subcommand = self.command_manager.find_command(argv)
+        except ValueError as err:
+            if self.options.debug:
+                raise
+            else:
+                LOG.error(err)
+            return 2
         cmd_factory, cmd_name, sub_argv = subcommand
         cmd = cmd_factory(self, self.options)
         err = None
